<?php

/** 
 * Joomla.Module - NetzSinn Social Share
 * ------------------------------------------------------------------------
 * @package     mod_bw_social_share
 * @author      Netzsinn.de
 * @copyright   2024 netzsinn.de
 * @license     GNU/GPLv3 <http://www.gnu.org/licenses/gpl-3.0.de.html>
 * @link        https://netzsinn.de
 * ------------------------------------------------------------------------
 */

// No direct access to this file
defined('_JEXEC') or die;

use Joomla\CMS\Installer\InstallerAdapter;
use Joomla\CMS\Installer\InstallerScriptInterface;
use Joomla\CMS\Language\Text;
use Joomla\CMS\Log\Log;
use Joomla\Registry\Registry;
use Joomla\Filesystem\File;

return new class () implements InstallerScriptInterface 
{
	private string $minimumJoomla = '4.4.0';
	private string $minimumPhp    = '7.4.0';

	/**
	 * Method to install the extension
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 */
	function install(InstallerAdapter $adapter): bool
	{
		echo Text::_('MOD_BW_SOCIAL_SHARE_INSTALLERSCRIPT_UNINSTALL');

		return true;
	}

	/**
	 * Method to uninstall the extension
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 */
	function uninstall(InstallerAdapter $adapter): bool
	{
		echo Text::_('MOD_BW_SOCIAL_SHARE_INSTALLERSCRIPT_UNINSTALL');

		return true;
	}

	/**
	 * Method to update the extension
	 *
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 */
	function update(InstallerAdapter $adapter): bool
	{
		$this->removeOldFiles();
		
		echo Text::_('MOD_BW_SOCIAL_SHARE_INSTALLERSCRIPT_UPDATE');
		
		return true;
	}

	/**
	 * Function called before extension installation/update/removal procedure commences
	 *
	 * @param   string            $type    The type of change (install, update or discover_install, not uninstall)
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 */
	public function preflight(string $type, InstallerAdapter $adapter): bool
    {
		// Check for the minimum PHP version before continuing
		if (!empty($this->minimumPhp) && version_compare(PHP_VERSION, $this->minimumPhp, '<')) {
			Log::add(Text::sprintf('JLIB_INSTALLER_MINIMUM_PHP', $this->minimumPhp), Log::WARNING, 'jerror');

			return false;
		}

		// Check for the minimum Joomla version before continuing
		if (!empty($this->minimumJoomla) && version_compare(JVERSION, $this->minimumJoomla, '<')) {
			Log::add(Text::sprintf('JLIB_INSTALLER_MINIMUM_JOOMLA', $this->minimumJoomla), Log::WARNING, 'jerror');

			return false;
		}

		echo Text::_('MOD_BW_SOCIAL_SHARE_INSTALLERSCRIPT_PREFLIGHT');
		echo Text::_('<p>Checks OK. Minimum Joomla: '.$this->minimumJoomla . ', Minimum PHP: ' . $this->minimumPhp . '</p>');

		return true;
	}

	/**
	 * Function called after extension installation/update/removal procedure commences
	 *
	 * @param   string            $type    The type of change (install, update or discover_install, not uninstall)
	 * @param   InstallerAdapter  $parent  The class calling this method
	 *
	 * @return  boolean  True on success
	 */
	public function postflight(string $type, InstallerAdapter $adapter): bool
    {
		#echo Text::_('MOD_BW_SOCIAL_SHARE_INSTALLERSCRIPT_POSTFLIGHT');

		return true;
	}


	private function removeOldFiles()
    {
        $files = [
            JPATH_ROOT . '/media/mod_bw_social_share/css/rrssb.css',
            JPATH_ROOT . '/media/mod_bw_social_share/js/_rrssb.min.js',
            JPATH_ROOT . '/media/mod_bw_social_share/js/rrssb.min.js',
            JPATH_ROOT . '/media/mod_bw_social_share/js/rrssb.js',
        ];

        foreach ($files as $file) {
            if (file_exists($file)) {
                File::delete($file);
				echo Text::_('<p>deleted file: ' . $file . '</p>');	

            }
        }
    }
};